%% compute heat kernel signature of an .off model
close all; clear all; clc

% input
filename = 'bunny.off';

%% compute hks
disp('read .off file and construct Laplacian matrix')
tic
[ cotWeiMat, massMat,Verts, Tris ] = yehan_OFFReader(filename);
toc

disp('start eigen decomposition')
tic
nev = min(300, size(Verts,1) - 1);
[eigvec, eigval] = eigs(cotWeiMat,massMat, nev, -1e-5);
eigval = diag(eigval);
toc

disp('create time step and compute corresponding HKS')
tic
tmin = abs(4*log(10) / eigval(end));
tmax = abs(4*log(10) / eigval(2));
nstep = 100;

stepsize = (log(tmax) - log(tmin)) / nstep;
logts = log(tmin):stepsize:log(tmax);
ts = exp(logts);
hks = eigvec(:, 2:end).^2 * exp( - abs(eigval(2:end)) * ts);
toc

%% plot
% hks at small time step
figure
trimesh(Tris,Verts(:,1),Verts(:,3),Verts(:,2),hks(:,3),'FaceColor','interp','edgecolor','interp');
colormap jet
colorbar
axis equal

figure
trimesh(Tris,Verts(:,1),Verts(:,3),Verts(:,2),hks(:,3),'FaceColor','interp','edgecolor','interp');
colormap prism
colorbar
axis equal

% hks at large time step
disp('plot')
figure
trimesh(Tris,Verts(:,1),Verts(:,3),Verts(:,2),hks(:,100),'FaceColor','interp','edgecolor','interp');
colormap jet
colorbar
axis equal

figure
trimesh(Tris,Verts(:,1),Verts(:,3),Verts(:,2),hks(:,100),'FaceColor','interp','edgecolor','interp');
colormap prism
colorbar
axis equal

%% scaled hks for pointwise comparison
scaledhks = abs( eigvec(:, 2:end) ).^2 * exp( ( abs(eigval(2)) - abs(eigval(2:end)) )  * ts);
colsum = sum(massMat*scaledhks);
scale = 1.0./ colsum; 
scaleMat = sparse(1:length(scale), 1:length(scale), scale);
scaledhks = scaledhks * scaleMat;

