%% read .off file, and compute cotangent-weight Laplacian matrix and mass matrix 
function [ cotWeiMat, massMat, Verts, Tris ] = yehan_OFFReader( filename )

%read file
fid = fopen(filename,'r');
if( fid==-1 )
    error('Cannot open file.');
    return;
end

str = fgets(fid);
if ~strcmp(str(1:3), 'OFF')
    error('Not a valid .OFF file.');    
end

str = fgets(fid);
[token, str] = strtok(str); nVert = str2num(token);
[token, ~]   = strtok(str); nTri  = str2num(token);

% read verts
[Verts, count] = fscanf(fid,'%f %f %f', 3*nVert);
if count ~= 3*nVert
    warning('Problem in reading vertices.');
end
Verts = transpose(reshape(Verts,3,nVert));

%read tris
[Tris, count] = fscanf(fid,'%d %d %d %d\n', 4*nTri);
if count ~= 4*nTri
    warning('Problem in reading faces.');
end
Tris = reshape(Tris, 4, nTri);
Tris = Tris' + 1;
Tris(:,1) = [];

fclose(fid);

%construct cotangent-weight Laplacian matrix and mass matrix
cotWeiMat = zeros(nVert);
massMat = zeros(1,nVert);
cotList = zeros(nTri,3);
for i = 1:nTri
    
    t1 = Tris(i,1);
    t2 = Tris(i,2);
    t3 = Tris(i,3);
    v1 = Verts(t1,:);
    v2 = Verts(t2,:);
    v3 = Verts(t3,:);
    
    area = 0.5 * norm(cross(v2 - v1, v3 - v1));
    if area == 0
        disp('There exists 0 area triangle\n')
    end
    
    a = norm(v3 - v2);
    b = norm(v1 - v3);
    c = norm(v2 - v1);
    
    cotList(i,1) = (b*b + c*c - a*a) / 4 / area;
    cotList(i,2) = (a*a + c*c - b*b) / 4 / area;
    cotList(i,3) = (b*b + a*a - c*c) / 4 / area;
    
    massMat(t1) = massMat(t1) + area/3;
    massMat(t2) = massMat(t2) + area/3;
    massMat(t3) = massMat(t3) + area/3;
end

for i = 1:nTri
    tri = Tris(i,:);
    cotan = cotList(i,:);
    idx = [1,2,3,1,2];
    for j  = 1:3
        cotWeiMat(tri(idx(j)),tri(idx(j))) = ...
            cotWeiMat(tri(idx(j)),tri(idx(j))) - 0.5 * (cotan(idx(j+1)) + cotan(idx(j+2)));
        cotWeiMat(tri(idx(j)),tri(idx(j+1))) = ...
            cotWeiMat(tri(idx(j)),tri(idx(j+1))) + 0.5 * cotan(idx(j+2));
        cotWeiMat(tri(idx(j)),tri(idx(j+2))) = ...
            cotWeiMat(tri(idx(j)),tri(idx(j+2))) + 0.5 * cotan(idx(j+1)); 
    end
end

massMat = sparse(1:nVert, 1:nVert, massMat);
cotWeiMat = sparse(cotWeiMat);
end

